@@ -1,17 +1,40 @@
-from typing import Tuple
+from typing import List, Optional, Tuple
 
 from rfa_toolbox.graphs import EnrichedNetworkNode
 
 try:
     import torch
-    import torchvision
-
-    from rfa_toolbox.vizualize import visualize_architecture
 except ImportError:
     pass
+
 from rfa_toolbox.encodings.pytorch.intermediate_graph import Digraph
 
 
+def _check_white_list(submodule_type, fq_submodule_name, classes_to_visit):
+    return (
+        classes_to_visit is None
+        and (
+            not fq_submodule_name.startswith("torch.nn")
+            or fq_submodule_name.startswith("torch.nn.modules.container")
+        )
+    ) or (
+        classes_to_visit is not None
+        and (
+            submodule_type in classes_to_visit or fq_submodule_name in classes_to_visit
+        )
+    )
+
+
+def _check_black_list(submodule_type, fq_submodule_name, classes_to_not_visit):
+    return (classes_to_not_visit is None) or (
+        classes_to_not_visit is not None
+        and (
+            submodule_type not in classes_to_not_visit
+            and fq_submodule_name not in classes_to_not_visit
+        )
+    )
+
+
 def make_graph(
     mod,
     classes_to_visit=None,
@@ -21,6 +44,7 @@ def make_graph(
     input_preds=None,
     parent_dot=None,
     ref_mod=None,
+    classes_to_not_visit=None,
 ):
     """
     This code was adapted from this blog article:
@@ -31,10 +55,12 @@ def make_graph(
     def find_name(i, self_input, suffix=None):
         if i == self_input:
             return suffix
-        cur = i.node().s("name")
+        try:
+            cur = i.node().s("name")
+        except RuntimeError:
+            return suffix + "-unknownType"
         if suffix is not None:
             cur = cur + "." + suffix
-            # print(cur)
         of = next(i.node().inputs())
         return find_name(of, self_input, suffix=cur)
 
@@ -147,42 +173,35 @@ def make_graph(
 
             if classes_found is not None:
                 classes_found.add(fq_submodule_name)
-            if (
-                classes_to_visit is None
-                and (
-                    not fq_submodule_name.startswith("torch.nn")
-                    or fq_submodule_name.startswith("torch.nn.modules.container")
-                )
-            ) or (
-                classes_to_visit is not None
-                and (
-                    submodule_type in classes_to_visit
-                    or fq_submodule_name in classes_to_visit
-                )
+            if _check_white_list(
+                submodule_type, fq_submodule_name, classes_to_visit
+            ) and _check_black_list(
+                submodule_type, fq_submodule_name, classes_to_not_visit
             ):
                 # go into subgraph
                 sub_prefix = prefix + submodule_name + "."
-                # with dot.subgraph(name="cluster_" + name) as sub_dot:
-                # sub_dot.attr(label=label)
-                submod = mod
-                # iterate to the lowest submodule hirarchy
-                for i, k in enumerate(submodule_name.split(".")):
-                    submod = getattr(submod, k)
-                    # create subgraph for the submodule
-                make_graph(
-                    submod,
-                    dot=dot,
-                    prefix=sub_prefix,
-                    input_preds=[preds[i] for i in list(n.inputs())[1:]],
-                    parent_dot=dot,
-                    classes_to_visit=classes_to_visit,
-                    classes_found=classes_found,
-                )
-                # creating a mapping from the c-values
-                # to the output of the respective subgraph
-                for i, o in enumerate(n.outputs()):
-                    # print(i, sub_prefix + f'out_{i}', type(o))
-                    preds[o] = {sub_prefix + f"out_{i}"}, set()
+                with dot.subgraph(name="cluster_" + name) as sub_dot:
+                    # sub_dot.attr(label=label)
+                    submod = mod
+                    # iterate to the lowest submodule hirarchy
+                    for i, k in enumerate(submodule_name.split(".")):
+                        submod = getattr(submod, k)
+                        # create subgraph for the submodule
+                    make_graph(
+                        submod,
+                        dot=sub_dot,
+                        prefix=sub_prefix,
+                        input_preds=[preds[i] for i in list(n.inputs())[1:]],
+                        parent_dot=dot,
+                        classes_to_visit=classes_to_visit,
+                        classes_found=classes_found,
+                        classes_to_not_visit=classes_to_not_visit,
+                    )
+                    # creating a mapping from the c-values
+                    # to the output of the respective subgraph
+                    for i, o in enumerate(n.outputs()):
+                        # print(i, sub_prefix + f'out_{i}', type(o))
+                        preds[o] = {sub_prefix + f"out_{i}"}, set()
             else:
                 # here the basic node (Conv2D, BatchNorm etc.) are created.
                 dot.node(name, label=label, shape="box")
@@ -229,7 +248,7 @@ def make_graph(
                 "aten::stack",
                 "aten::reshape",
                 "aten::split_with_sizes",
-                "aten::cat",
+                # "aten::cat",
                 "aten::expand",
                 "aten::expand_as",
                 "aten::_shape_as_tensor",
@@ -272,7 +291,9 @@ def make_graph(
 
 
 def create_graph_from_model(
-    model: torch.nn.Module, input_res: Tuple[int, int, int, int] = (1, 3, 399, 399)
+    model: torch.nn.Module,
+    input_res: Tuple[int, int, int, int] = (1, 3, 399, 399),
+    custom_layers: Optional[List[str]] = None,
 ) -> EnrichedNetworkNode:
     """Create a graph of enriched network nodes from a PyTorch-Model.
 
@@ -283,15 +304,15 @@ def create_graph_from_model(
                         color_channels, height, width) for CNNs.
                         Needs to be a 2-Tuple of shape (batch_size,
                         num_features) for fully connected networks.
+        custom_layers:  Class-names of custom layers, like DropPath
+        or Involutions, which are not part of
+                        torch.nn. Keep in mind that unknown layers
+                        will defaulted to have no effect on the
+                        receptive field size. You may need to
+                        implement some additional layer handlers.
 
     Returns:
         The EnrichedNetworkNodeGraph
     """
     tm = torch.jit.trace(model, (torch.randn(*input_res),))
-    return make_graph(tm, ref_mod=model).to_graph()
-
-
-if __name__ == "__main__":
-    model = torchvision.models.googlenet()
-    graph = create_graph_from_model(model)
-    visualize_architecture(graph, "inceptionv3", input_res=32).view()
+    return make_graph(tm, ref_mod=model, classes_to_not_visit=custom_layers).to_graph()
